#!/bin/bash

function safeexec () {
    eval "$@"
    if [ "$?" != 0 ] ; then
	echo "Error when executing \`$@'"
	exit 1
    fi
}

echo "________________________________________________________________________________"
echo "Searching for ocaml-3.10.2"
until [ -d "ocaml-3.10.2" ] ; do
    if tar xfz ocaml-3.10.2.tar.gz || tar xjf ocaml-3.10.2.tar.bz2 ; then
	break;
    fi
    echo "I can't find ocaml-3.10.2"
    echo "you can try getting http://caml.inria.fr/pub/distrib/ocaml-3.10/ocaml-3.10.2.tar.gz"
    exit 1
    break
done


echo "________________________________________________________________________________"
echo "Creating thread library"
sleep 1
safeexec cp -r ocaml-3.10.2/otherlibs/systhreads .
cd systhreads;
safeexec patch -p1 <../PATCHES/systhreads_patch;
cd ..;
echo "________________________________________________________________________________"
echo "Creating runtime"
sleep 1;
mkdir runtime
safeexec cp -r ocaml-3.10.2/asmrun runtime/
safeexec cp -r ocaml-3.10.2/byterun runtime/
cd runtime;
safeexec patch -p1 <../PATCHES/runtime_patch;
echo "________________________________________________________________________________"
cd ../ocaml-3.10.2;
echo "compiling ocaml with new code generator"
sleep 1
safeexec patch -p1 <../PATCHES/gencode_patch;
safeexec ./configure --prefix $PWD/../out; 
safeexec cp -r config ../runtime/
cd ../systhreads;
safeexec make libthreadsnat.a
cd ../runtime;
safeexec cp ../systhreads/posix_n.o asmrun/;
make;
safeexec cd ../ocaml-3.10.2;

echo "source patched"
sleep 1

make world;
safeexec chmod 777 -R ../ocaml-3.10.2;
safeexec make world opt install;
cd ..;
safeexec cp runtime/asmrun/libasmrun.a out/lib/ocaml
safeexec cp systhreads/libthreadsnat.a out/lib/ocaml
echo "________________________________________________________________________________"
echo "compiling gc"
sleep 1
cd runtime/gcs
safeexec make
cd ../..
echo "source compiled"
echo "________________________________________________________________________________"

