
let rec intervalle r a b =
  if a > b then r
  else intervalle2 (a::r) (a+1) b

and intervalle2 r a b =
  if a > b then []
  else intervalle (a::r) (a+1) b

let intervalle a b = intervalle [] a b 


module SafeThread :
sig
  type t
  val create : ('a -> 'b) -> 'a -> t
  val join : t -> unit
end =
struct
  type t = {
    thread : Thread.t;
    id : int;
  }

  let generate_id = 
    let cpt = ref 0 in
      fun () ->
	cpt := succ !cpt;
	!cpt

  module S = Set.Make(struct type t = int let compare = (-) end)

  let threads = ref S.empty

  let register_thread id =
    Printf.printf "REGISTER_THREAD_<%d>\n" id;
    threads := S.add id !threads
    
  (** return true if thread is supposed to be alive ; false elseway *)
  let check_thread id =
(*    Printf.printf "CHECK_THREAD_<%d>\n" id; *)
    S.mem id !threads

  let unregister_thread id =
    Printf.printf "UNREGISTER_THREAD_<%d>\n" id;
    threads := S.remove id !threads

  let create f a =
    let id = generate_id () in
    let () = register_thread id in
    let f = fun () ->
      try  f a ; unregister_thread id
      with anyexception ->
	unregister_thread id ;
	raise anyexception
    in
    let th = Thread.create f () in
      Printf.printf "THREAD_<%d>_CREATED\n" id;
      { thread = th ;
	id = id }

  let join th =
    (try Thread.join th.thread with _ -> ());
    let wait = ref true in
      while !wait do
	begin
	  if check_thread th.id then
	    ()
	  else
	    wait := false
	end;
	begin
	  try
	    (* we are not supposed to enter this loop for long unless
	       Thread.join is boggus *)
	    (* Thread.delay 0.1 *)
	    ignore(intervalle 1 2)
	  with _ -> ()
	end
      done;
      Printf.printf "THREAD_<%d>_JOINED\n" th.id;
end



let _ =
  let size = 10_000 in
  let v1 = Array.make size 0 in
  let v2 = Array.make size 0 in
  let v3 = Array.make size 0 in
  let v4 = Array.make size 0 in
  let v5 = Array.make size 0 in
  let () = print_endline "Create threads" in
    List.iter
      SafeThread.join
      begin
	List.map 
	  (fun v2 ->
	     SafeThread.create
	       (fun () ->
		  for i = 0 to size - 1 do
		    v2.(i) <- i;
		    intervalle 0 (10_000 + (i*i) mod 10_000)
		  done;
	       )
	       ())
	  [v1;v2;v3;v4;v5]
      end;
    Array.iter
      (fun v1 ->
	 Array.iteri
	   (fun i v ->
	      if i > 0 && i < size
	      then
		if v = v1.(i-1) + 1 then
		  ()
		else
		  (Printf.printf "problem with an array, at index %d" i ;
		   assert false)
	   )
	   v1
      )
      [| v1;v2;v3;v4;v5 |]
    ;
    print_endline "intervalle 0 1000 : start";
    for i = 0 to 100_000 do
      ignore(intervalle 0 1000);
    done;
    print_endline "intervalle 0 1000 : end"; 
    print_endline "Exit !"
