(**
  * 
  *  test pour OC4MC 
  *
  *  jeu de la vie tir? de DAOC 
  * 
  * $Id: life.ml,v 1.1 2009-04-29 11:15:48 emmanuel Exp $ 
  *
  **)

(* 
    le d?but de la hi?rarchie de classes n'est pas touch?
*)

class cell a =
  object
    val mutable v = (a : bool)
    method isAlive = v
  end ;;


class virtual absWorld n m  =
  object(self)
    val mutable tcell = Array.make_matrix n m (new cell false)
    val maxx = n
    val maxy = m
    val mutable gen = 0
    method private draw(c) =
     if c#isAlive then print_string "*"
     else print_string "."
    method display() =
      for i = 0 to (maxx-1) do
        for j=0 to (maxy -1) do
          print_string " " ;
          self#draw(tcell.(i).(j))
        done ;
        print_newline()
      done
    method getCell(i,j) = tcell.(i).(j)
    method setCell(i,j,c) = tcell.(i).(j) <- c
    method getCells = tcell
    method countCells () = 
      let r = ref 0 in 
        for i = 0 to (maxx - 1) do 
          for j = 0 to (maxy - 1) do 
            if tcell.(i).(j)#isAlive then incr r
          done
        done;
        !r
  end ;;


class world n m =
  object(self)
    inherit absWorld n m
    method neighbors(x,y) =
      let r = ref 0 in
      for i=x-1 to x+1 do
        let k = (i+maxx) mod maxx in
        for j=y-1 to y+1 do
          let l = (j + maxy) mod maxy in
            if tcell.(k).(l)#isAlive then incr r
        done
      done;
      if tcell.(x).(y)#isAlive then decr r ;
      !r

    method nextGen() =
      let w2 = new world maxx maxy in
      for i=0 to maxx-1 do
        for j=0 to maxy -1 do
          let n = self#neighbors(i,j) in
          if tcell.(i).(j)#isAlive
          then (if (n = 2) || (n = 3) then w2#setCell(i,j,new cell true))
          else (if n = 3 then w2#setCell(i,j,new cell true))
        done
      done ;
      tcell <- w2#getCells ;
      gen <- gen + 1
  end ;;


(* 
 * la classe threadedworld 
 *     h?rite de world, 
 *     modifie nextGen
 *     ajoute une m?thode nextLines
 *
 *)

class threadedworld n m nbth = 
object(self)
  val maxth = nbth   
  inherit world n m   

   method nextLines w2 infb  supb = 
      for i = 0 to maxx-1 do  
        for j=infb to supb do
          let n = self#neighbors(i,j) in
          if tcell.(i).(j)#isAlive
          then (if (n = 2) || (n = 3) then w2#setCell(i,j,new cell true))
          else (if n = 3 then w2#setCell(i,j,new cell true))
        done
      done 

    method nextGen() =
      let w2 = new threadedworld maxx maxy maxth in

      let nbLines = maxy / maxth in
      let l = ref [] in 
      let infb = ref 0 and supb = ref (nbLines - 1) in 
        for i = 1 to maxth-1 do 
          let ib = !infb and sb = !supb in 
          let th = 
            Thread.create (fun () -> self#nextLines w2 ib sb ) () in 
          l:= th :: !l;
          infb := !infb + nbLines;
          supb := !supb + nbLines
        done; 
        let th = 
          Thread.create (fun () -> self#nextLines w2 !infb (maxy-1) ) () in  
                 l:= th :: !l ;

        List.iter Thread.join !l;

        tcell <- w2#getCells ;
        gen <- gen + 1

end


(* 
 * quelques tests 
 *)

let size = int_of_string Sys.argv.(1) ;;
let nbgen = int_of_string Sys.argv.(2);;
let nb_th = int_of_string Sys.argv.(3) ;; 


let main w ngen df  =

  w#setCell(4,4,new cell true) ;
  w#setCell(4,5,new cell true) ;
  w#setCell(4,6,new cell true) ;
  
    for i=1 to ngen do 
       print_int i;
       print_newline();
       w#nextGen()
    done;
     if df then Printf.printf "#Cells = %d\n" (w#countCells());;



 (*main (new world 100 120) 1000 true ;;
*)
 main (new threadedworld size size nb_th) nbgen true;;

